/**
 * Copyright (c) 2017 committers of YAKINDU and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * 	committers of YAKINDU - initial API and implementation
 * 
 */
package org.yakindu.training.client;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import org.yakindu.training.RuntimeService;
import org.yakindu.training.TimerService;
import org.yakindu.training.lightimpl.LightStatemachine;

@SuppressWarnings("serial")
public class LightControllerDemo extends JFrame {

	private Light light;

	private JButton lightSwitch;

	protected void setupStatemachine() {

		TimerService timer = new TimerService();

		LightStatemachine statemachine = new LightStatemachine();

		statemachine.setTimer(timer);
		statemachine.getSCIWire().setSCIWireOperationCallback(power -> light.setBrightness(power));

		statemachine.init();
		statemachine.enter();

		RuntimeService.getInstance().registerStatemachine(statemachine, 100);
		lightSwitch.addActionListener(e -> statemachine.getSCIButton().raiseDown());
	}

	protected void createUI() {
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());
		light = new Light();
		getContentPane().add(BorderLayout.CENTER, light);
		lightSwitch = new JButton();
		lightSwitch.setText("light_switch");
		getContentPane().add(BorderLayout.SOUTH, lightSwitch);
		setSize(200, 200);
		setVisible(true);
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				LightControllerDemo application = new LightControllerDemo();
				application.createUI();
				application.setupStatemachine();
			}
		});
	}

	public static class Light extends JPanel {

		public void setBrightness(Long brightness) {
			switch (brightness.intValue()) {
			case 0:
				setBackground(Color.black);
				break;
			case 50:
				setBackground(Color.yellow.darker());
				break;
			case 100:
				setBackground(Color.yellow.brighter());
				break;
			}
		}
	}

}
